"use strict";
/**
 * Copyright © 2019 kevinpollet <pollet.kevin@gmail.com>`
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE.md file.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dockerode_1 = __importDefault(require("dockerode"));
const path_1 = require("path");
const split2_1 = __importDefault(require("split2"));
const stream_1 = require("stream");
const createDockerBuildContext_1 = require("./createDockerBuildContext");
const getBuildConfig_1 = require("./config/getBuildConfig");
const overrideBuildConfig_1 = require("./config/overrideBuildConfig");
exports.buildImage = (dir, options = {}) => __awaiter(void 0, void 0, void 0, function* () {
    const absoluteDir = path_1.resolve(process.cwd(), dir);
    const defaultBuildConfig = yield getBuildConfig_1.getBuildConfig(absoluteDir);
    const buildConfigWithOverrides = overrideBuildConfig_1.overrideBuildConfig(defaultBuildConfig, options);
    const dockerBuildContext = yield createDockerBuildContext_1.createDockerBuildContext(absoluteDir, buildConfigWithOverrides);
    const dockerImageTags = (buildConfigWithOverrides.tags || []).map(tag => `${buildConfigWithOverrides.name}:${tag}`);
    const getDaemonMessage = new stream_1.Transform({
        writableObjectMode: true,
        transform(chunk, _, callback) {
            const { stream, error } = chunk; // eslint-disable-line @typescript-eslint/no-explicit-any
            callback(error ? new Error(error) : null, stream || null);
        },
    });
    return new dockerode_1.default()
        .buildImage(dockerBuildContext, {
        t: dockerImageTags,
        buildargs: { AUTH_TOKEN: process.env.AUTH_TOKEN },
    })
        .then(daemonStream => daemonStream.pipe(split2_1.default(line => JSON.parse(line))).pipe(getDaemonMessage));
});
